Package: terra
Type: Package
Title: Spatial Data Analysis
Version: 1.9-25
Date: 2026-05-04
Depends: R (>= 3.5.0), methods
Suggests: parallel, tinytest, ncdf4, sf (>= 0.9-8), deldir, XML,
        leaflet (>= 2.2.1), htmlwidgets, future, future.apply
LinkingTo: Rcpp
Imports: Rcpp (>= 1.0-10)
SystemRequirements: C++17, GDAL (>= 2.2.3), GEOS (>= 3.4.0), PROJ (>=
        4.9.3), TBB, sqlite3
Encoding: UTF-8
Language: en-US
Maintainer: Robert J. Hijmans <r.hijmans@gmail.com>
Description: Methods for spatial data analysis with vector (points, lines, polygons) and raster (grid) data. Methods for vector data include geometric operations such as intersect and buffer. Raster methods include local, focal, global, zonal and geometric operations. The predict and interpolate methods facilitate the use of regression type (interpolation, machine learning) models for spatial prediction, including with satellite remote sensing data. Processing of very large files is supported. See the manual and tutorials on <https://rspatial.org/> to get started.
License: GPL (>= 3)
URL: https://rspatial.org/, https://rspatial.github.io/terra/
BugReports: https://github.com/rspatial/terra/issues
LazyLoad: yes
Authors@R: c(
	person("Robert J.", "Hijmans", role=c("cre", "aut"),  
			email="r.hijmans@gmail.com", comment=c(ORCID="0000-0001-5872-2872")),			
    person("Andrew", "Brown", role="aut", comment=c(ORCID="0000-0002-4565-533X")),
    person("Márcia", "Barbosa", role="aut", comment=c(ORCID="0000-0001-8972-7713")),
    person("Krzysztof", "Dyba", role="ctb", comment=c(ORCID="0000-0002-8614-3816")),
    person("Roger", "Bivand", role="ctb", comment=c(ORCID="0000-0003-2392-6140")),
    person("Michael", "Chirico", role="ctb", comment=c(ORCID="0000-0003-0787-087X")),
    person("Emanuele", "Cordano", role="ctb",comment=c(ORCID="0000-0002-3508-5898")),
    person("Edzer", "Pebesma", role="ctb", comment=c(ORCID="0000-0001-8049-7069")),
    person("Barry", "Rowlingson", role="ctb", comment=c(ORCID="0000-0002-8586-6625")),
    person("Michael D.", "Sumner", role="ctb", comment=c(ORCID="0000-0002-2471-7511")))
NeedsCompilation: yes
Packaged: 2026-05-04 17:42:14 UTC; rhijm
Author: Robert J. Hijmans [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-5872-2872>),
  Andrew Brown [aut] (ORCID: <https://orcid.org/0000-0002-4565-533X>),
  Márcia Barbosa [aut] (ORCID: <https://orcid.org/0000-0001-8972-7713>),
  Krzysztof Dyba [ctb] (ORCID: <https://orcid.org/0000-0002-8614-3816>),
  Roger Bivand [ctb] (ORCID: <https://orcid.org/0000-0003-2392-6140>),
  Michael Chirico [ctb] (ORCID: <https://orcid.org/0000-0003-0787-087X>),
  Emanuele Cordano [ctb] (ORCID: <https://orcid.org/0000-0002-3508-5898>),
  Edzer Pebesma [ctb] (ORCID: <https://orcid.org/0000-0001-8049-7069>),
  Barry Rowlingson [ctb] (ORCID: <https://orcid.org/0000-0002-8586-6625>),
  Michael D. Sumner [ctb] (ORCID:
    <https://orcid.org/0000-0002-2471-7511>)
Repository: CRAN
Date/Publication: 2026-05-06 07:50:02 UTC
