# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

.have_TBB <- function() {
    .Call(`_terra_have_TBB`)
}

.open_file_limit <- function() {
    .Call(`_terra_open_file_lim`)
}

.proj_conf_test <- function() {
    .Call(`_terra_proj_conf_test`)
}

proj_version <- function() {
    .Call(`_terra_proj_version`)
}

hex2rgb <- function(s) {
    .Call(`_terra_hex2rgb`, s)
}

rgb2hex <- function(x) {
    .Call(`_terra_rgb2hex`, x)
}

.sameSRS <- function(x, y) {
    .Call(`_terra_sameSRS`, x, y)
}

.SRSinfo <- function(s) {
    .Call(`_terra_getCRSname`, s)
}

.getLinearUnits <- function(s) {
    .Call(`_terra_getLinearUnits`, s)
}

.geotransform <- function(fname) {
    .Call(`_terra_geotransform`, fname)
}

.gdal_setconfig <- function(option, value) {
    invisible(.Call(`_terra_gdal_setconfig`, option, value))
}

.gdal_getconfig <- function(option) {
    .Call(`_terra_gdal_getconfig`, option)
}

.gdalinfo <- function(filename, options, oo) {
    .Call(`_terra_ginfo`, filename, options, oo)
}

.gdalmdinfo <- function(filename, options) {
    .Call(`_terra_gmdinfo`, filename, options)
}

.sdinfo <- function(filename) {
    .Call(`_terra_sd_info`, filename)
}

.gdal_version <- function() {
    .Call(`_terra_gdal_version`)
}

.geos_version <- function(runtime = FALSE, capi = FALSE) {
    .Call(`_terra_geos_version`, runtime, capi)
}

.metadata <- function(filename) {
    .Call(`_terra_metatdata`, filename)
}

.sdsmetadata <- function(filename) {
    .Call(`_terra_sdsmetatdata`, filename)
}

.parsedsdsmetadata <- function(filename) {
    .Call(`_terra_sdsmetatdataparsed`, filename)
}

.gdaldrivers <- function() {
    .Call(`_terra_gdal_drivers`)
}

.set_gdal_warnings <- function(level) {
    invisible(.Call(`_terra_set_gdal_warnings`, level))
}

.seedinit <- function(seed_val) {
    invisible(.Call(`_terra_seed_init`, seed_val))
}

.gdalinit <- function(projpath, datapath) {
    invisible(.Call(`_terra_gdal_init`, projpath, datapath))
}

.precRank <- function(x, y, minc, maxc, tail) {
    .Call(`_terra_percRank`, x, y, minc, maxc, tail)
}

.clearVSIcache <- function(vsi) {
    invisible(.Call(`_terra_clearVSIcache`, vsi))
}

.setGDALCacheSizeMB <- function(x, vsi) {
    invisible(.Call(`_terra_setGDALCacheSizeMB`, x, vsi))
}

.getGDALCacheSizeMB <- function(vsi) {
    .Call(`_terra_getGDALCacheSizeMB`, vsi)
}

.get_proj_search_paths <- function() {
    .Call(`_terra_get_proj_search_paths`)
}

.set_proj_search_paths <- function(paths, with_proj = FALSE) {
    .Call(`_terra_set_proj_search_paths`, paths, with_proj)
}

.PROJ_network <- function(enable, url) {
    .Call(`_terra_PROJ_network`, enable, url)
}

.proj_pipelines <- function(source_crs, target_crs, authority, AOI, use, grid_availability, desired_accuracy, strict_containment, axis_order_authority_compliant) {
    .Call(`_terra_proj_pipelines`, source_crs, target_crs, authority, AOI, use, grid_availability, desired_accuracy, strict_containment, axis_order_authority_compliant)
}

.removeDriver <- function(d) {
    invisible(.Call(`_terra_removeDriver`, d))
}

.pearson <- function(x, y, narm) {
    .Call(`_terra_pearson_cor`, x, y, narm)
}

.weighted_pearson <- function(x, y, weights, narm = TRUE) {
    .Call(`_terra_weighted_pearson_cor`, x, y, weights, narm)
}

.unique_symmetric_rows <- function(x, y) {
    .Call(`_terra_uniqueSymmetricRows`, x, y)
}

.arnames <- function(filename, filter) {
    .Call(`_terra_arnames`, filename, filter)
}

.dimfo <- function(filename, array_name) {
    .Call(`_terra_dimfo`, filename, array_name)
}

dist2segmentPoint_geo <- function(plon, plat, lon1, lat1, lon2, lat2, ilon, ilat) {
    .Call(`_terra_dist2segmentPoint_geo`, plon, plat, lon1, lat1, lon2, lat2, ilon, ilat)
}

intermediate <- function(lon1, lat1, lon2, lat2, n, distance) {
    .Call(`_terra_intermediate`, lon1, lat1, lon2, lat2, n, distance)
}

